/*
 * Read and write "certdb" files, which are just text files containing
 * enaml lines.
 *
 * Copyright (C) 2001  Robey Pointer
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _CERTDB_H_
#define _CERTDB_H_

typedef struct {
        char *hostname;
        GList *alias;
        int length;
        void *cert;
        int refcount;
} Say2Cert;


Say2Cert *say2_cert_new (void);
Say2Cert *say2_cert_new_host (const char *hostname, int length, const void *cert_data);
void say2_cert_add_alias (Say2Cert *cert, const char *hostname);
void say2_cert_free (Say2Cert *cert);
gboolean say2_cert_matches_host (Say2Cert *cert, const char *hostname);

GHashTable *say2_cert_table_new (void);
void say2_cert_table_free (GHashTable *table);
void say2_cert_table_add (GHashTable *table, Say2Cert *cert);
Say2Cert *say2_cert_table_lookup (GHashTable *table, const char *hostname);
GList *say2_cert_table_get_hostnames (GHashTable *table);

gboolean say2_cert_table_load (GHashTable *table, const char *filename);
gboolean say2_cert_table_save_fd (GHashTable *table, int fd);
gboolean say2_cert_table_save (GHashTable *table, const char *filename);

#endif	/* _CERTDB_H_ */
