/*
 * functions for running as a daemon (and dropping root privs, etc)
 *
 * Copyright (C) 2001  Robey Pointer <robey@lag.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include "log.h"
#include "daemon.h"

#ifdef OPEN_MAX
#define LAST_FD OPEN_MAX
#else
#define LAST_FD 1024
#endif

void
daemonize (const char *name)
{
        pid_t child;
        int i;
        FILE *fp;
        char *filename;

	child = fork ();
	if (child < 0) {
                log ("CANNOT FORK INTO THE BACKGROUND -- BAILING OUT");
                exit (1);
	}

        if (child != 0) {
                /* parent */
                exit (0);
        }

        /* child: get a new session id */
        setsid ();

        child = fork ();
        if (child < 0) {
                log ("CANNOT FORK INTO THE BACKGROUND II -- BAILING OUT");
                exit (1);
        }

        if (child != 0) {
                /* parent */
                exit (0);
        }

        /* child again: now we have a different session id from process id */

        chdir ("/");
        umask (0);
        setpgid (0, 0);

        /* leave stderr open ... */
        close (0); close (1);
        for (i = 3; i < LAST_FD; i++) {
                close(i);
        }

        if (geteuid () == 0) {
                filename = g_strdup_printf ("/var/run/%s.pid", name);
                fp = fopen (filename, "wt");
                fprintf (fp, "%u\n", (unsigned) getpid ());
                fclose (fp);
                g_free (filename);
        }
}

/* drop down to running as a normal user */
void
daemon_drop_privs (const char *username)
{
        struct passwd *pwentry;
        uid_t uid;

        if (geteuid () != 0) {
                /* not running as root */
                return;
        }

        /* might be a uid #? */
        uid = (uid_t) strtoul (username, NULL, 10);
        if (uid != 0) {
                pwentry = getpwuid (uid);
        } else {
                pwentry = getpwnam (username);
        }

        if (pwentry == NULL) {
                log ("UNABLE TO FIND UID FOR %s -- BAILING OUT", username);
                exit (1);
        }
        log ("dropping root privs to run as %s", pwentry->pw_name);

        /* this should set the uid, euid, suid, and fsuid all at once */
        setuid (pwentry->pw_uid);
}
