/* $Id: rcfile.c,v 1.2 2001/05/02 05:06:28 robey Exp $
 *
 * Read a config file of key/value pairs and make callbacks (or set local
 * variables).
 *
 * Copyright (C) 2000  Robey Pointer <robey@lag.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>
#include "rcfile.h"
#include "log.h"

#define MAX_LINE       	512

static gboolean
rcfile_read_file (const char *filename, RCCallbackFn fn, int depth)
{
        FILE *fp;
        char buffer[MAX_LINE];
        char *p, *line;
        gboolean ret;

        fp = fopen (filename, "r");
        if (fp == NULL) {
                log ("*** Unable to read config file: %s", filename);
                return FALSE;
        }

        ret = TRUE;
        while (! feof (fp) && ret) {
                fgets (buffer, MAX_LINE, fp);
                if (feof (fp)) {
                        break;
                }
                buffer[MAX_LINE-1] = 0;

                line = buffer;
                while (*line && (*line <= ' ')) {
                        line++;
                }
                while ((strlen (buffer) > 0) && (buffer[strlen (buffer)-1] <= ' ')) {
                        buffer[strlen (buffer)-1] = '\0';
                }

                if (!*line || (*line == ';') || (*line == '#')) {
                        continue;
                }

                if (*line == '$') {
                        /* special command */
                        if (strncmp (line, "$include ", 9) == 0) {
                                if (depth > 10) {
                                        log ("*** Include file nested too deep: %s", filename);
                                        ret = FALSE;
                                        continue;
                                }
                                line += 9;
                                while (*line && (*line <= ' ')) {
                                        line++;
                                }
                                if (*line) {
                                        ret = rcfile_read_file (line, fn, depth+1);
                                        continue;
                                }
                        }
                }

                p = strchr (line, ':');
                if (p == NULL) {
                        /* key, no value */
                        ret = (*fn) (line, NULL);
                } else {
                        *p++ = '\0';
                        while (*p && (*p <= ' ')) {
                                p++;
                        }
                        while ((strlen (line) > 0) && (line[strlen (line)-1] <= ' ')) {
                                line[strlen (line)-1] = '\0';
                        }
                        ret = (*fn) (line, p);
                }
        }

        fclose (fp);
        return ret;
}

gboolean
rcfile_read (const char *filename, RCCallbackFn fn)
{
        return rcfile_read_file (filename, fn, 0);
}
