/* $Id: sock-ssl.c,v 1.7 2001/07/12 23:29:01 robey Exp $
 *
 * Handles the ugliness of using the openSSL library.
 *
 * Copyright (C) 2000 Eazel, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Authors: Robey Pointer <robey@eazel.com>
 *          Mike Fleming <mfleming@eazel.com>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* only build this file if openssl is installed */
#ifdef HAVE_OPENSSL

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#include <glib.h>

#include "log.h"
#include "sock-ssl.h"


/* verify server's cert?  unverified cert -> failed connect */
#define VERIFY_CERTS

/* dump debug info about the ciphers used? */
#undef DEBUG_CIPHER_LIST

/* log cipher & # bits every time a connection is made? */
#define SOCKET_SSL_VERBOSE


/* store paths to look for CA certs */
static char *ssl_cert_directory = NULL;
static char *ssl_cert_file = NULL;


#ifdef VERIFY_CERTS
static int
verify_callback(int state, X509_STORE_CTX *x509_context)
{
	char name[256];
	int err;

#if 1
        X509_NAME_oneline (X509_get_subject_name (x509_context->current_cert), name, sizeof(name));
        log ("SSL cert %s from: %s", state ? "valid" : "invalid", name);
#endif

	err = X509_STORE_CTX_get_error (x509_context);
	if (! state) {
		X509_NAME_oneline (X509_get_subject_name (x509_context->current_cert), name, sizeof(name));
		/* Remote site specified a certificate, but it's not correct */
		log ("SSL cert error: %s: %s", X509_verify_cert_error_string(err), name);
		/* reject */
		return 0;
	}

	/* Accept connection */
	return 1;
}
#endif	/* VERIFY_CERTS */


/* stolen from openssl */
static void
ssl_log_error(const char *prefix)
{
	unsigned long l;
	char buf[200];
	const char *file, *data;
	int line, flags;

	l = ERR_get_error_line_data (&file, &line, &data, &flags);
	if (l) {
		log ("%s%s: %s", prefix, ERR_error_string (l, buf),
		     (flags & ERR_TXT_STRING) ? data : "");
	} else {
		log ("%s(no ssl error)", prefix);
	}
}


/* initialize the openssl library and set up some callbacks and paths.
 *     cert_directory: a directory where CA certs can be found
 *     cert_file: a single CA cert file
 * either cert_directory or cert_file may be NULL.  if both are NULL, then
 * no certs are automatically loaded for checking connections.
 *
 * generally you will just pass in the path to the cert file that contains
 * the public key for the CA that signed the upstream web server's key.
 * for example, for the apache-modssl demo key set, you would use the
 * snakeoil-ca-rsa.crt file.
 *
 * you can add certs to a socket later, before switching to SSL mode:
 * socket_set_der_cert() will attach a CA cert to a socket, to be used
 * in cert verification once SSL starts on that socket.
 *
 * if no certs are given, client-mode SSL will always fail because it will
 * be unable to verify the server's cert.  you could #undef VERIFY_CERTS to
 * get around this, but you will lose all auth checking, so don't do it.
 * also, i don't think SSL's symmetric encryption is secure when no certs
 * are used.
 */
gboolean
socket_init_ssl (char *cert_directory, char *cert_file)
{
        char buffer[256];
        const char *filename;

        /* load strings for better error messages */
	SSL_load_error_strings ();
        /* initialize SSL methods */
	OpenSSL_add_ssl_algorithms ();

        if (cert_directory != NULL) {
                ssl_cert_directory = g_strdup (cert_directory);
        }
        if (cert_file != NULL) {
                ssl_cert_file = g_strdup (cert_file);
        }

        /* this garbage is to load any innate "randomness" that's missing from your life */
        /* SSL seems to work without this, although it claims it might not work "as well" */
        filename = RAND_file_name (buffer, sizeof (buffer));
        if ((filename == NULL) || !RAND_load_file (filename, -1)) {
                if (RAND_status () == 0) {
                        log ("openssl: unable to load random state");
                }
        }

	return TRUE;
}

/* this is the saddest part of all.  look away, children!
 * openssl seems to require us to pre-seed the "TMP DH" (i won't ask) with our own set of
 * meaningless arbitrary bytes, if we want to be a server.  why?!?!
 */

static unsigned char what_the_fuck[]={
	0xDA,0x58,0x3C,0x16,0xD9,0x85,0x22,0x89,0xD0,0xE4,0xAF,0x75,
	0x6F,0x4C,0xCA,0x92,0xDD,0x4B,0xE5,0x33,0xB8,0x04,0xFB,0x0F,
	0xED,0x94,0xEF,0x9C,0x8A,0x44,0x03,0xED,0x57,0x46,0x50,0xD3,
	0x69,0x99,0xDB,0x29,0xD7,0x76,0x27,0x6B,0xA2,0xD3,0xD4,0x12,
	0xE2,0x18,0xF4,0xDD,0x1E,0x08,0x4C,0xF6,0xD8,0x00,0x3E,0x7C,
	0x47,0x74,0xE8,0x33,
	};
static unsigned char what_the_fuck_jr[]={ 0x02 };

static gboolean
unbelievable_openssl_hack (SSL_CTX *ssl_context)
{
	DH *dh = NULL;

	if ((dh=DH_new()) == NULL) {
                return FALSE;
        }
	dh->p = BN_bin2bn (what_the_fuck, sizeof (what_the_fuck), NULL);
	dh->g = BN_bin2bn (what_the_fuck_jr, sizeof (what_the_fuck_jr), NULL);
	if ((dh->p == NULL) || (dh->g == NULL)) {
                DH_free (dh);
                return FALSE;
        }

        SSL_CTX_set_tmp_dh (ssl_context, dh);
	DH_free (dh);
        return TRUE;
}

/* in order to possibly associate a different CA list with each connection, each connection
 * needs to have its own context.
 */
static SSL_CTX *
socket_ssl_new_context (SocketSSLType which)
{
        SSL_CTX *ssl_context;

	/* apparently this can never error */
        if (which == SOCKET_SSL_CLIENT) {
                ssl_context = SSL_CTX_new (SSLv3_client_method ());
        } else if (which == SOCKET_SSL_SERVER) {
                ssl_context = SSL_CTX_new (SSLv23_server_method ());
        } else {
                log ("FATAL: socket_ssl_new_context called with bad witch");
                exit (1);
        }

	if (! ssl_context) {
		ssl_log_error ("FATAL: openssl context creation failed: ");
		exit (1);
	}

	/* set verify level on certs */
	if (! SSL_CTX_set_default_verify_paths (ssl_context)) {
		ssl_log_error ("FATAL: openssl default configuration failed: ");
		exit (1);
	}

        if (ssl_cert_file != NULL || ssl_cert_directory != NULL) {
                if (! SSL_CTX_load_verify_locations (ssl_context, ssl_cert_file,
                                                     ssl_cert_directory)) {
                        ssl_log_error ("FATAL: openssl cert loading failed: ");
                        exit(1);
                }
        }

#ifdef VERIFY_CERTS
	SSL_CTX_set_verify (ssl_context, SSL_VERIFY_PEER, verify_callback);
#endif

        if (which == SOCKET_SSL_SERVER) {
                if (! unbelievable_openssl_hack (ssl_context)) {
                        ssl_log_error ("SSL: TMP DH hack failed: ");
                }
        }

        log ("cert store: %p", ssl_context->cert_store);
        return ssl_context;
}


/* turn off SSL ciphers that have patent issues */
static int
socket_turn_off_evil_ciphers(SSL *ssl_handle)
{
	STACK_OF(SSL_CIPHER) *stack;
	char *rule = NULL, *newbuf;
	const char *cipher_name;
	int i;

	stack = SSL_get_ciphers (ssl_handle);
	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
		cipher_name = SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i));
		/* IDEA & RC5: patented till longer */
		/* RC4: questionable (leaked trade secret?) */
                if ((strstr (cipher_name, "IDEA") != NULL) ||
		    (strstr (cipher_name, "RC4") != NULL) ||
		    (strstr (cipher_name, "RC5") != NULL)) {
			if (rule != NULL) {
				newbuf = g_strconcat (rule, ":-", cipher_name, NULL);
				g_free(rule);
				rule = newbuf;
			} else {
				rule = g_strconcat ("DEFAULT:-", cipher_name, NULL);
			}
		}
	}
	/* stack is not allocated, don't free it. */

#ifdef DEBUG_CIPHER_LIST
	log ("SSL cipher list rule: '%s'", rule ? rule : "(none)");
#endif
	if (rule && (! SSL_set_cipher_list (ssl_handle, rule))) {
		ssl_log_error ("error: ssl_set_cipher_list: ");
		return SOCKET_SSL_ERROR_OPENSSL;
	}

#ifdef DEBUG_CIPHER_LIST
	/* debug list */
	stack = SSL_get_ciphers (ssl_handle);
	log ("SSL cipher list: (%d entries)", sk_SSL_CIPHER_num (stack));
	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
		log ("stack %d: %s", i, SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i)));
	}
#endif

	return 0;
}

static X509 *
slurp_cert_from_der_buffer (const void *buffer, int len)
{
        BIO *in;
        X509 *cert = NULL;

        in = BIO_new_mem_buf ((void *)buffer, len);
        if (in == NULL) {
                log ("X509_read_buffer: malloc error!");
                return NULL;
        }
        if (d2i_X509_bio (in, &cert) == NULL) {
                goto fail;
        }

fail:
        if (in != NULL) {
                BIO_free (in);
        }
        return cert;
}

static EVP_PKEY *
slurp_key_from_pem_buffer (const char *buffer, int len)
{
        BIO *in;
        EVP_PKEY *key = NULL;

        in = BIO_new_mem_buf ((void *)buffer, len);
        if (in == NULL) {
                log ("openssl: slurp_key_from_file: malloc error!");
                return NULL;
        }
        /* for future coolness: 3rd & 4th params below are a callback function and userdata for
         * the callback.  this would let us fetch a password from ??somewhere?? to decode the
         * private key, if the private key is password-protected.
         */
        key = PEM_read_bio_PrivateKey (in, NULL, NULL, NULL);
        if (key == NULL) {
                log ("openssl: unable to read private key!");
                goto fail;
        }

fail:
        if (in != NULL) {
                BIO_free (in);
        }
        return key;
}

static gboolean
socket_ssl_add_CA_from_buffer (SSL *ssl_handle, const char *der_buffer, int der_length)
{
        X509 *cert;
        SSL_CTX *ssl_context;

        g_return_val_if_fail (ssl_handle != NULL, FALSE);

        cert = slurp_cert_from_der_buffer (der_buffer, der_length);
        if (cert != NULL) {
                ssl_context = SSL_get_SSL_CTX (ssl_handle);
                X509_STORE_add_cert (SSL_CTX_get_cert_store (ssl_context), cert);
        }
        return (cert != NULL ? TRUE : FALSE);
}

/* for some reason, when you're the server, you must do it this way */
static gboolean
socket_ssl_load_cert_from_buffer (SSL *ssl_handle, const char *der_buffer, int der_length)
{
        X509 *cert;

        g_return_val_if_fail (ssl_handle != NULL, FALSE);

        cert = slurp_cert_from_der_buffer (der_buffer, der_length);
        if (cert == NULL) {
                log ("openssl: can't read server cert from der buffer");
                return FALSE;
        }
        if (! SSL_use_certificate (ssl_handle, cert)) {
                ssl_log_error ("openssl: bad server cert: ");
                return FALSE;
        }
        return TRUE;
}

static gboolean
socket_ssl_load_private_key (SSL *ssl_handle, const char *pem_buffer, int pem_length)
{
        EVP_PKEY *key;

        g_return_val_if_fail (ssl_handle != NULL, FALSE);

        key = slurp_key_from_pem_buffer (pem_buffer, pem_length);
        if (key == NULL) {
                log ("openssl: can't read private key from pem buffer");
                return FALSE;
        }
        if (! SSL_use_PrivateKey (ssl_handle, key)) {
                ssl_log_error ("openssl: bad private key: ");
                return FALSE;
        }
        return TRUE;
}

void
socket_ssl_free (SSL *ssl_handle)
{
        SSL_CTX *ssl_context;

        ssl_context = SSL_get_SSL_CTX (ssl_handle);
        SSL_free (ssl_handle);
        SSL_CTX_free (ssl_context);
}

/* given a normal socket that's just been connected somewhere, attempt to open an SSL
 * session.
 * a buffer containing a DER cert can be included, which will add that CA to the
 * list for this socket (under the assumption that you know that the socket will use
 * exactly that CA).
 * on success, returns an SSL handle which can be used to do SSL_read/SSL_write.
 */
int
socket_ssl_begin_ssl (Socket *sock, int fd, SocketSSLType which)
{
	SSL *ssl_handle;
	int err = 0;
	int bits, alg_bits;
	X509 *server_cert;

	ssl_handle = SSL_new (socket_ssl_new_context (which));
	if (ssl_handle == NULL) {
		ssl_log_error ("error: SSL_new: ");
		err = SOCKET_SSL_ERROR_OPENSSL;
		goto out;
	}

        if (sock->ssl_cert_length > 0) {
                if (which == SOCKET_SSL_SERVER) {
                        if (! socket_ssl_load_cert_from_buffer (ssl_handle, sock->ssl_cert_buffer,
                                                                sock->ssl_cert_length)) {
                                err = SOCKET_SSL_ERROR_BAD_CA;
                                goto bad;
                        }
                } else {
                        if (! socket_ssl_add_CA_from_buffer (ssl_handle, sock->ssl_cert_buffer,
                                                             sock->ssl_cert_length)) {
                                log ("SSL: socket's attached CA was invalid :(");
                                err = SOCKET_SSL_ERROR_BAD_CA;
                                goto bad;
                        }
                }
        }

        if (sock->ssl_key_length > 0) {
                if (! socket_ssl_load_private_key (ssl_handle, sock->ssl_key_buffer,
                                                   sock->ssl_key_length)) {
                        err = SOCKET_SSL_ERROR_BAD_KEY;
                        goto bad;
                }
                if (! SSL_check_private_key (ssl_handle)) {
                        ssl_log_error ("SSL: server key/cert mismatch: ");
                        err = SOCKET_SSL_ERROR_BAD_KEY;
                        goto bad;
                }
        }

	err = SSL_set_fd (ssl_handle, fd);
	if (err <= 0) {
		ssl_log_error ("error: SSL_set_fd: ");
		err = SOCKET_SSL_ERROR_OPENSSL;
		goto bad;
	}

	err = socket_turn_off_evil_ciphers (ssl_handle);
	if (err != 0) {
		goto bad;
	}

        if (which == SOCKET_SSL_CLIENT) {
                err = SSL_connect (ssl_handle);
                if (err != 1) {
                        ssl_log_error ("error: SSL_connect: ");
                        err = SOCKET_SSL_ERROR_OPENSSL;
                        goto bad;
                }
        } else {
                /* server */
                err = SSL_accept (ssl_handle);
                if (err != 1) {
                        ssl_log_error ("error: SSL_accept: ");
                        err = SOCKET_SSL_ERROR_OPENSSL;
                        goto bad;
                }
        }

	bits = SSL_get_cipher_bits (ssl_handle, &alg_bits);
	if (bits <= 0) {
		log ("error: SSL not supported by server");
		err = SOCKET_SSL_ERROR_NOT_SUPPORTED;
		goto bad;
	}

#ifdef SOCKET_SSL_VERBOSE
	log ("SSL connect: cipher %s, %d bits", SSL_get_cipher_name (ssl_handle), bits);
#endif

	server_cert = SSL_get_peer_certificate (ssl_handle);
	if ((which == SOCKET_SSL_CLIENT) && (server_cert == NULL)) {
		ssl_log_error ("error: no server cert: ");
		err = SOCKET_SSL_ERROR_NO_CERT;
		goto bad;
	}

        sock->ssl_handle = ssl_handle;

out:
	return 0;

bad:
	SSL_free (ssl_handle);
	return err;
}


#endif	/* HAVE_OPENSSL */
