/* $Id: sock-ssl.h,v 1.6 2001/07/12 23:29:01 robey Exp $
 *
 * Prototypes and definitions for the proxy server's SSL wrappers.
 *
 * Copyright (C) 2000  Eazel, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef _SOCK_SSL_H_
#define _SOCK_SSL_H_

#include "sock.h"

typedef enum {
	SOCKET_SSL_CLIENT = 1,
	SOCKET_SSL_SERVER = 2
} SocketSSLType;

/* errors that might be returned by socket_ssl_begin_ssl() */
#define SOCKET_SSL_ERROR_OPENSSL	1	/* some internal openssl library error */
#define SOCKET_SSL_ERROR_NOT_SUPPORTED	2	/* peer doesn't support encryption (0 bits) */
#define SOCKET_SSL_ERROR_NO_CERT	3	/* server doesn't have any certificate */
#define SOCKET_SSL_ERROR_BAD_CA		4	/* cert CA was attached by client, but invalid */
#define SOCKET_SSL_ERROR_BAD_KEY	5	/* private key file couldn't be loaded */
#define SOCKET_SSL_NO_SSL		99	/* compiled without SSL support */

#ifdef HAVE_OPENSSL
gboolean socket_init_ssl (char *cert_directory, char *cert_file);
void socket_ssl_free (SSL *ssl_handle);
int socket_ssl_begin_ssl (Socket *sock, int fd, SocketSSLType which);
#else
#define socket_init_ssl(dir, file) FALSE
#define socket_ssl_free(ssl)
#define socket_ssl_begin_ssl(sock, fd, which) SOCKET_SSL_NO_SSL
#endif	/* HAVE_OPENSSL */

#endif	/* _SOCK_SSL_H_ */
