#ifndef _SQL_H_
#define _SQL_H_

#ifdef HAVE_MYSQL
#define Socket SOCKETBLOB
#include <mysql.h>
#undef Socket
#endif

typedef struct {
#ifdef HAVE_MYSQL
        MYSQL mysql;
        MYSQL *connection;
#endif
        char *host;
        char *user;
        char *pass;
        char *db;
        time_t last_tried;	/* time we last tried to reconnect */
} SQLState;

typedef enum {
        SQL_OK = 0,
        SQL_SLEEPING,		/* server is down; waiting before retrying */
        SQL_EXISTS,		/* adding entry that already exists */
        SQL_INTERNAL = 98,
        SQL_OTHER = 99
} SQLError;

const char *sql_error_string (SQLError err);
void sql_free_list (GList *list);
char *sql_quote_buffer (const char *in, int len);
char *sql_quote_string (const char *in);
char *sql_unquote_buffer (const char *in, int *len);
char *sql_unquote_string (const char *in);
SQLState *sql_init (const char *host, const char *user, const char *pass, const char *db);
SQLError sql_query (SQLState *state, const char *query, GList **out);
SQLError sql_update (SQLState *state, const char *query, int *rows);
void sql_shutdown (SQLState *state);

/* seconds to wait before retrying a connect to the SQL server */
#define WAIT_BETWEEN_TRIES	60

#endif	/* _SQL_H_ */
