/*
 * test socket stuff
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "log.h"
#include "sock.h"
#include "sock-ssl.h"
#include "dnscache.h"

static void
connect_callback (Socket *sock, int success)
{
        if (! success) {
                fprintf (stderr, "*** Failed to connect to server.\n");
                exit (3);
        }
        fprintf (stderr, "Connected.  Starting SSL ...\n");
        if (! socket_begin_ssl (sock, SOCKET_SSL_CLIENT)) {
                fprintf (stderr, "*** Unable to negotiate SSL.\n");
                exit (4);
        }
        printf ("Now using ssl.\n");
}

static void
accept_callback (Socket *sock, void *user_data, const char *host, int port)
{
        fprintf (stderr, "New connection!\n");
        if (! socket_begin_ssl (sock, SOCKET_SSL_SERVER)) {
                fprintf (stderr, "*** Unable to negotiate SSL.\n");
                exit (5);
        }
        printf ("Now using ssl.\n");
}

static gboolean
read_cert_file (const char *filename, char **buffer, int *length)
{
        int fd;
        struct stat statbuf;

        *buffer = NULL;
        *length = 0;

        if (stat (filename, &statbuf) != 0) {
                return FALSE;
        }

        fd = open (filename, O_RDONLY);
        if (fd < 0) {
                return FALSE;
        }

        *buffer = g_malloc (statbuf.st_size);
        *length = statbuf.st_size;
        if (read (fd, *buffer, *length) < *length) {
                g_free (*buffer);
                *buffer = NULL;
                *length = 0;
                close (fd);
                return FALSE;
        }

        close (fd);
        return TRUE;
}

int
main (int argc, char **argv)
{
        GMainLoop *l_main_loop = NULL;
        Socket *sock, *sock2;
        char *buffer;
        int length;

        log_set (stderr);

        if (! read_cert_file ("/u/robey/openssl/cert.der", &buffer, &length)) {
                fprintf (stderr, "*** Couldn't read the cert file.\n");
                exit (1);
        }

        if (! socket_init_ssl (NULL, NULL)) {
		fprintf (stderr, "*** Compiled without SSL support.\n");
		exit (1);
	}

        if (argv[1][0] == 's') {
                sock = socket_new (NULL, 6666);
        } else if (argv[1][0] == 'r') {
                /* resolve stuff */
                const char *hostname;
                struct in_addr addr;

                if (dnscache_lookup (&addr, argv[2]) != 0) {
                        fprintf (stderr, "*** Can't lookup '%s'\n", argv[2]);
                        exit (1);
                }
                printf ("IP: %s\n", inet_ntoa (addr));
                if (dnscache_lookup_reverse (&addr, &hostname) != 0) {
                        fprintf (stderr, "*** Can't reverse lookup '%s'\n", inet_ntoa (addr));
                        exit (1);
                }
                printf ("reverse: %s\n", hostname);
                exit (0);
        } else {
                sock = socket_new (NULL, 0);
                sock2 = socket_new (NULL, 0);
        }

        if (sock == NULL) {
                fprintf (stderr, "*** Unable to create initial socket.\n");
                exit (1);
        }

        if (! read_cert_file ("/u/robey/openssl/cert.der", &buffer, &length)) {
                fprintf (stderr, "*** Couldn't read the cert file.\n");
                exit (1);
        }
        socket_set_cert (sock, buffer, length);
        if (argv[1][0] != 's') {
                socket_set_cert (sock2, buffer, length);
        }
        g_free (buffer);

        if (argv[1][0] == 's') {
                if (! read_cert_file ("/u/robey/openssl/key", &buffer, &length)) {
                        fprintf (stderr, "*** Couldn't read the private key file.\n");
                        exit (1);
                }
                socket_set_private_key (sock, buffer, length);
                g_free (buffer);

                if (! socket_listen (sock, accept_callback)) {
                        fprintf (stderr, "*** Can't listen at this port\n");
                        exit (6);
                }
        } else {
                if (! socket_connect (sock, "localhost", 6666, connect_callback)) {
                        fprintf (stderr, "*** Localhost lookup failed.\n");
                        exit (2);
                }
                if (! socket_connect (sock2, "localhost", 6666, connect_callback)) {
                        fprintf (stderr, "*** Localhost lookup failed.\n");
                        exit (2);
                }
        }

	l_main_loop = g_main_new (TRUE);
	g_main_run (l_main_loop);

	g_main_destroy (l_main_loop);

        exit (0);
}
